// *******************************************************************
// Copyright 2000-2010 iAnywhere Solutions, Inc.  All rights reserved.
// *******************************************************************
#ifndef SSERROR_H_INCLUDED
#define SSERROR_H_INCLUDED

#include "sqltype.h"
#ifdef UNDER_PALM_OS
    #include "uldef.h"
#else
    #include <stdlib.h>
#endif


typedef enum {
    STREAM_ERROR_NONE                                                = 0,
    STREAM_ERROR_PARAMETER                                           = 1,
    STREAM_ERROR_PARAMETER_NOT_UINT32                                = 2,
    STREAM_ERROR_PARAMETER_NOT_UINT32_RANGE                          = 3,
    STREAM_ERROR_PARAMETER_NOT_BOOLEAN                               = 4,
    STREAM_ERROR_PARAMETER_NOT_HEX                                   = 5,
    STREAM_ERROR_MEMORY_ALLOCATION                                   = 6,
    STREAM_ERROR_PARSE                                               = 7,
    STREAM_ERROR_READ                                                = 8,
    STREAM_ERROR_WRITE                                               = 9,
    STREAM_ERROR_END_WRITE                                           = 10,
    STREAM_ERROR_END_READ                                            = 11,
    STREAM_ERROR_NOT_IMPLEMENTED                                     = 12,
    STREAM_ERROR_WOULD_BLOCK                                         = 13,
    STREAM_ERROR_GENERATE_RANDOM                                     = 14,
    STREAM_ERROR_INIT_RANDOM                                         = 15,
    STREAM_ERROR_SEED_RANDOM                                         = 16,
    STREAM_ERROR_CREATE_RANDOM_OBJECT                                = 17,
    STREAM_ERROR_SHUTTING_DOWN                                       = 18,
    STREAM_ERROR_DEQUEUING_CONNECTION                                = 19,
    STREAM_ERROR_SECURE_CERTIFICATE_ROOT                             = 20,
    STREAM_ERROR_SECURE_CERTIFICATE_COMPANY_NAME                     = 21,
    STREAM_ERROR_SECURE_CERTIFICATE_NOT_TRUSTED                      = 24,
    STREAM_ERROR_SECURE_DUPLICATE_CONTEXT                            = 25,
    STREAM_ERROR_SECURE_SET_IO                                       = 26,
    STREAM_ERROR_SECURE_CERTIFICATE_FILE_NOT_FOUND                   = 33,
    STREAM_ERROR_SECURE_READ_CERTIFICATE                             = 34,
    STREAM_ERROR_SECURE_READ_PRIVATE_KEY                             = 35,
    STREAM_ERROR_SECURE_SET_PRIVATE_KEY                              = 36,
    STREAM_ERROR_SECURE_CERTIFICATE_EXPIRY_DATE                      = 37,
    STREAM_ERROR_SECURE_EXPORT_CERTIFICATE                           = 38,
    STREAM_ERROR_SECURE_ADD_CERTIFICATE                              = 39,
    STREAM_ERROR_SECURE_TRUSTED_CERTIFICATE_FILE_NOT_FOUND           = 40,
    STREAM_ERROR_SECURE_TRUSTED_CERTIFICATE_READ                     = 41,
    STREAM_ERROR_SECURE_CERTIFICATE_COUNT                            = 42,
    STREAM_ERROR_SECURE_CREATE_CERTIFICATE                           = 43,
    STREAM_ERROR_SECURE_IMPORT_CERTIFICATE                           = 44,
    STREAM_ERROR_SECURE_ADD_TRUSTED_CERTIFICATE                      = 48,
    STREAM_ERROR_SECURE_CREATE_PRIVATE_KEY_OBJECT                    = 49,
    STREAM_ERROR_SECURE_CERTIFICATE_EXPIRED                          = 50,
    STREAM_ERROR_SECURE_CERTIFICATE_COMPANY_UNIT                     = 51,
    STREAM_ERROR_SECURE_CERTIFICATE_COMMON_NAME                      = 52,
    STREAM_ERROR_SECURE_HANDSHAKE                                    = 53,
    STREAM_ERROR_HTTP_VERSION                                        = 54,
    STREAM_ERROR_SOCKET_HOST_NAME_NOT_FOUND                          = 57,
    STREAM_ERROR_SOCKET_CREATE_TCPIP                                 = 58,
    STREAM_ERROR_SOCKET_CREATE_UDP                                   = 59,
    STREAM_ERROR_SOCKET_BIND                                         = 60,
    STREAM_ERROR_SOCKET_CLEANUP                                      = 61,
    STREAM_ERROR_SOCKET_CLOSE                                        = 62,
    STREAM_ERROR_SOCKET_CONNECT                                      = 63,
    STREAM_ERROR_SOCKET_GET_NAME                                     = 64,
    STREAM_ERROR_SOCKET_GET_OPTION                                   = 65,
    STREAM_ERROR_SOCKET_SET_OPTION                                   = 66,
    STREAM_ERROR_SOCKET_LISTEN                                       = 67,
    STREAM_ERROR_SOCKET_SHUTDOWN                                     = 68,
    STREAM_ERROR_SOCKET_SELECT                                       = 69,
    STREAM_ERROR_SOCKET_STARTUP                                      = 70,
    STREAM_ERROR_SOCKET_LOCALHOST_NAME_NOT_FOUND                     = 71,
    STREAM_ERROR_SOCKET_GET_HOST_BY_ADDR                             = 72,
    STREAM_ERROR_LOAD_NETWORK_LIBRARY                                = 73,
    STREAM_ERROR_SOCKET_PORT_OUT_OF_RANGE                            = 74,
    STREAM_ERROR_ACTSYNC_NO_PORT                                     = 75,
    STREAM_ERROR_ACTSYNC_NOT_INSTALLED                               = 76,
    STREAM_ERROR_HTTP_CONTENT_TYPE_NOT_SPECIFIED                     = 77,
    STREAM_ERROR_HTTP_CLIENT_ID_NOT_SET                              = 78,
    STREAM_ERROR_HTTP_BUFFER_SIZE_OUT_OF_RANGE                       = 79,
    STREAM_ERROR_HTTP_EXTRA_DATA_END_READ                            = 80,
    STREAM_ERROR_HTTP_CRLF_ENCODED_MISSING                           = 81,
    STREAM_ERROR_HTTP_CRLF_MISSING                                   = 82,
    STREAM_ERROR_HTTP_NO_CONTD_CONNECTION                            = 83,
    STREAM_ERROR_HTTP_CHUNK_LEN_ENCODED_MISSING                      = 84,
    STREAM_ERROR_HTTP_CHUNK_LEN_BAD_CHARACTER                        = 85,
    STREAM_ERROR_HTTP_BAD_STATUS_CODE                                = 86,
    STREAM_ERROR_HTTP_UNKNOWN_TRANSFER_ENCODING                      = 87,
    STREAM_ERROR_HTTP_UNABLE_TO_PARSE_COOKIE                         = 88,
    STREAM_ERROR_HTTP_EXPECTED_POST                                  = 89,
    STREAM_ERROR_SOCKET_LIVENESS_OUT_OF_RANGE                        = 200,
    STREAM_ERROR_READ_TIMEOUT                                        = 201,
    STREAM_ERROR_WRITE_TIMEOUT                                       = 202,
    STREAM_ERROR_DUN_NOT_SUPPORTED                                   = 203,
    STREAM_ERROR_DUN_DIAL_FAILED                                     = 204,
    STREAM_ERROR_SECURE_NO_SERVER_CERTIFICATE                        = 205,
    STREAM_ERROR_SECURE_NO_SERVER_CERTIFICATE_PASSWORD               = 206,
    STREAM_ERROR_SECURE_NO_TRUSTED_ROOTS                             = 207,
    STREAM_ERROR_SECURE_REDUNDANT_SERVER_CERTIFICATE_PASSWORD        = 208,
    STREAM_ERROR_HTTP_AUTHENTICATION_REQUIRED                        = 209,
    STREAM_ERROR_HTTP_PROXY_AUTHENTICATION_REQUIRED                  = 210,
    STREAM_ERROR_HTTP_AUTHENTICATION_FAILED                          = 211,
    STREAM_ERROR_HTTP_PROXY_AUTHENTICATION_FAILED                    = 212,
    STREAM_ERROR_HTTP_NO_USERID                                      = 213,
    STREAM_ERROR_HTTP_NO_PASSWORD                                    = 214,
    STREAM_ERROR_HTTP_UNSUPPORTED_AUTH_ALGORITHM                     = 215,
    STREAM_ERROR_HTTP_HEADER_PARSE_ERROR                             = 216,
    STREAM_ERROR_HTTP_SERVER_AUTH_FAILED                             = 217,
    STREAM_ERROR_INTERRUPTED                                         = 218,
    STREAM_ERROR_HTTP_INVALID_CHARACTER                              = 219,
    STREAM_ERROR_INTERNAL                                            = 220,
    STREAM_ERROR_SECURE_OPEN_SYSTEM_CERT_STORE                       = 221,
    STREAM_ERROR_SECURE_IMPORT_CERT_FROM_SYSTEM_STORE                = 222,
    STREAM_ERROR_SECURE_NO_CERTS_IN_SYS_STORE                        = 223,
    STREAM_ERROR_LOAD_LIBRARY_FAILURE                                = 224,
    STREAM_ERROR_LIBRARY_ENTRY_POINT_NOT_FOUND                       = 225,
    STREAM_ERROR_INVALID_SYNC_PROTOCOL                               = 226,
    STREAM_ERROR_INTERNAL_PROTOCOL_NOT_LOADED                        = 227,
    STREAM_ERROR_SERVER_ERROR                                        = 228,
    STREAM_ERROR_MISSING_PARAMETER                                   = 229,
    STREAM_ERROR_COULD_NOT_OPEN_FILE_FOR_WRITE                       = 230,
    STREAM_ERROR_PROTOCOL_ERROR                                      = 231,
    STREAM_ERROR_INVALID_COMPRESSION_TYPE                            = 232,
    STREAM_ERROR_VALUE_OUT_OF_RANGE                                  = 233,
    STREAM_ERROR_UNEXPECTED_HTTP_REQUEST_TYPE                        = 234,
    STREAM_ERROR_HTTP_MALFORMED_SESSION_COOKIE                       = 235,
    STREAM_ERROR_HTTP_INTERNAL_HEADER_STATE                          = 236,
    STREAM_ERROR_HTTP_INTERNAL_REQUEST_TYPE                          = 237,
    STREAM_ERROR_INTERNAL_API                                        = 238,
    STREAM_ERROR_NO_ECC_FIPS                                         = 239,
    STREAM_ERROR_UNRECOGNIZED_TLS_TYPE                               = 240,
    STREAM_ERROR_CONNECT_TIMEOUT                                     = 241,
    STREAM_ERROR_INCONSISTENT_FIPS                                   = 242,
    STREAM_ERROR_INVALID_LOCAL_PATH                                  = 243,
    STREAM_ERROR_HTTP_INVALID_SESSION_KEY                            = 244,
    STREAM_ERROR_INVALID_NETWORK_LIBRARY                             = 245,
    STREAM_ERROR_NETWORK_LIBRARY_VERSION_MISMATCH                    = 248,
    STREAM_ERROR_AUTHENTICATION_FAILED                               = 249,
    STREAM_ERROR_SACI_IMPLEMENTATION_MISMATCH                        = 250,
    STREAM_ERROR_E2EE_UNKNOWN_PUBLIC_KEY_ENC_TYPE                    = 251,
    STREAM_ERROR_E2EE_UNEXPECTED_PUBLIC_KEY_ENC_TYPE                 = 252,
    STREAM_ERROR_E2EE_MISMATCHED_KEYS                                = 253,
    STREAM_ERROR_E2EE_UNEXPECTED_PRIVATE_KEY_TYPE                    = 254,
    STREAM_ERROR_E2EE_READING_PRIVATE_KEY                            = 255,
    STREAM_ERROR_E2EE_NO_PRIVATE_KEY_IN_FILE                         = 256,
    STREAM_ERROR_E2EE_DECODING_PRIVATE_KEY_FILE                      = 257,
    STREAM_ERROR_E2EE_READING_PRIVATE_KEY_FILE                       = 258,
    STREAM_ERROR_E2EE_MISSING_PRIVATE_KEY_PASSWORD                   = 259,
    STREAM_ERROR_E2EE_MISSING_PRIVATE_KEY                            = 260,
    STREAM_ERROR_E2EE_INVALID_TYPE                                   = 261,
    STREAM_ERROR_E2EE_INIT_ECC                                       = 262,
    STREAM_ERROR_E2EE_PUBLIC_KEY                                     = 263,
    STREAM_ERROR_COULD_NOT_OPEN_FILE                                 = 264,
    STREAM_ERROR_HTTP_UNKNOWN_SESSION                                = 265,
    STREAM_ERROR_SACI_ERROR                                          = 266,
    STREAM_ERROR_HTTP_FAILED_READING_HEADERS                         = 277,
    STREAM_ERROR_HTTP_BAD_REQUEST                                    = 278,
    STREAM_ERROR_CONNECTION_ABANDONED                                = 279,
    STREAM_ERROR_LAST                                                = 0x7FFF
} _ss_error_code;

typedef asa_uint16	    ss_error_code;


// All system errors are system-dependent, but zero is "no system error".
#define STREAM_SYSTEM_ERROR_NONE    0


#endif   /* SSERROR_H_INCLUDED */

